@@ -30,8 +30,11 @@ from .extradatasets import MockDataset, with_epoch, with_length
 from .filters import (
     associate,
     batched,
+    Cached,
+    LMDBCached,
     decode,
     detshuffle,
+    extract_keys,
     getfirst,
     info,
     map,
@@ -39,6 +42,7 @@ from .filters import (
     map_tuple,
     pipelinefilter,
     rename,
+    rename_keys,
     rsample,
     select,
     shuffle,
@@ -46,6 +50,7 @@ from .filters import (
     to_tuple,
     transform_with,
     unbatched,
+    xdecode,
 )
 from .handlers import (
     ignore_and_continue,
@@ -70,3 +75,5 @@ from .tariterators import tarfile_samples, tarfile_to_samples
 from .utils import PipelineStage, repeatedly
 from .writer import ShardWriter, TarWriter, numpy_dumps, torch_dumps
 from .mix import RandomMix, RoundRobin
+
+__vesion__ = ""
