@@ -6,19 +6,11 @@
 
 """Automatically decode webdataset samples."""
 
-import io
-import json
-import os
-import pickle
-import re
-import tempfile
+import io, json, os, pickle, re, tempfile
 from functools import partial
 
 import numpy as np
 
-from .checks import checkmember, checknotnone
-
-
 """Extensions passed on to the image decoder."""
 image_extensions = "jpg jpeg png ppm pgm pbm pnm".split()
 
@@ -34,12 +26,62 @@ def torch_loads(data):
     :param data: data to be decoded
     """
     import io
+
     import torch
 
     stream = io.BytesIO(data)
     return torch.load(stream)
 
 
+def tenbin_loads(data):
+    from . import tenbin
+
+    return tenbin.decode_buffer(data)
+
+
+def msgpack_loads(data):
+    import msgpack
+
+    return msgpack.unpackb(data)
+
+
+def npy_loads(data):
+    import numpy.lib.format
+
+    stream = io.BytesIO(data)
+    return numpy.lib.format.read_array(stream)
+
+
+def cbor_loads(data):
+    import cbor
+
+    return cbor.loads(data)
+
+
+decoders = {
+    "txt": lambda data: data.decode("utf-8"),
+    "text": lambda data: data.decode("utf-8"),
+    "transcript": lambda data: data.decode("utf-8"),
+    "cls": lambda data: int(data),
+    "cls2": lambda data: int(data),
+    "index": lambda data: int(data),
+    "inx": lambda data: int(data),
+    "id": lambda data: int(data),
+    "json": lambda data: json.loads(data),
+    "jsn": lambda data: json.loads(data),
+    "pyd": lambda data: pickle.loads(data),
+    "pickle": lambda data: pickle.loads(data),
+    "pth": lambda data: torch_loads(data),
+    "ten": tenbin_loads,
+    "tb": tenbin_loads,
+    "mp": msgpack_loads,
+    "msg": msgpack_loads,
+    "npy": npy_loads,
+    "npz": lambda data: np.load(io.BytesIO(data)),
+    "cbor": cbor_loads,
+}
+
+
 def basichandlers(key, data):
     """Handle basic file decoding.
 
@@ -60,48 +102,17 @@ def basichandlers(key, data):
     """
     extension = re.sub(r".*[.]", "", key)
 
-    if extension in "txt text transcript":
-        return data.decode("utf-8")
+    if extension in decoders:
+        return decoders[extension](data)
 
-    if extension in "cls cls2 class count index inx id".split():
-        try:
-            return int(data)
-        except ValueError:
-            return None
-
-    if extension in "json jsn":
-        return json.loads(data)
-
-    if extension in "pyd pickle".split():
-        return pickle.loads(data)
-
-    if extension in "pth".split():
-        return torch_loads(data)
-
-    if extension in "ten tb".split():
-        from . import tenbin
-
-        return tenbin.decode_buffer(data)
-
-    if extension in "mp msgpack msg".split():
-        import msgpack
-
-        return msgpack.unpackb(data)
-
-    if extension in "npy".split():
-        import numpy.lib.format
-
-        stream = io.BytesIO(data)
-        return numpy.lib.format.read_array(stream)
-
-    if extension in "npz".split():
-        return np.load(io.BytesIO(data))
+    return None
 
 
 ################################################################
 # Generic extension handler.
 ################################################################
 
+
 def call_extension_handler(key, data, f, extensions):
     """Call the function f with the given data if the key matches the extensions.
 
@@ -115,7 +126,7 @@ def call_extension_handler(key, data, f, extensions):
         target = target.split(".")
         if len(target) > len(extension):
             continue
-        if extension[-len(target):] == target:
+        if extension[-len(target) :] == target:
             return f(data)
     return None
 
@@ -194,7 +205,8 @@ class ImageHandler:
         :param imagespec: short string indicating the type of decoding
         :param extensions: list of extensions the image handler is invoked for
         """
-        checkmember(imagespec, list(imagespecs.keys()), "unknown image specification")
+        if imagespec not in list(imagespecs.keys()):
+            raise ValueError("Unknown imagespec: %s" % imagespec)
         self.imagespec = imagespec.lower()
         self.extensions = extensions
 
@@ -219,7 +231,8 @@ class ImageHandler:
             return img
         elif atype == "numpy":
             result = np.asarray(img)
-            checkmember(result.dtype, [np.uint8])
+            if result.dtype != np.uint8:
+                raise ValueError("ImageHandler: numpy image must be uint8")
             if etype == "uint8":
                 return result
             else:
@@ -228,7 +241,8 @@ class ImageHandler:
             import torch
 
             result = np.asarray(img)
-            checkmember(result.dtype, [np.uint8])
+            if result.dtype != np.uint8:
+                raise ValueError("ImageHandler: torch image must be uint8")
             if etype == "uint8":
                 result = np.array(result.transpose(2, 0, 1))
                 return torch.tensor(result)
@@ -354,13 +368,14 @@ class Decoder:
     handlers until some handler returns something other than None.
     """
 
-    def __init__(self, handlers, pre=None, post=None, only=None):
+    def __init__(self, handlers, pre=None, post=None, only=None, partial=False):
         """Create a Decoder.
 
         :param handlers: main list of handlers
         :param pre: handlers called before the main list (.gz handler by default)
         :param post: handlers called after the main list (default handlers by default)
         :param only: a list of extensions; when give, only ignores files with those extensions
+        :param partial: allow partial decoding (i.e., don't decode fields that aren't of type bytes)
         """
         if isinstance(only, str):
             only = only.split()
@@ -373,6 +388,7 @@ class Decoder:
         assert all(callable(h) for h in pre), f"one of {pre} not callable"
         assert all(callable(h) for h in post), f"one of {post} not callable"
         self.handlers = pre + handlers + post
+        self.partial = partial
 
     def decode1(self, key, data):
         """Decode a single field of a sample.
@@ -406,9 +422,15 @@ class Decoder:
             if self.only is not None and k not in self.only:
                 result[k] = v
                 continue
-            checknotnone(v)
-            assert isinstance(v, bytes)
-            result[k] = self.decode1(k, v)
+            assert v is not None
+            if self.partial:
+                if isinstance(v, bytes):
+                    result[k] = self.decode1(k, v)
+                else:
+                    result[k] = v
+            else:
+                assert isinstance(v, bytes)
+                result[k] = self.decode1(k, v)
         return result
 
     def __call__(self, sample):
