@@ -11,8 +11,12 @@ in webdataset.filters, and you can find IterableDataset wrappers in
 webdataset.processing.
 """
 
+import io
+from fnmatch import fnmatch
+import re
 import itertools, os, random, sys, time
 from functools import reduce, wraps
+import pickle
 
 import numpy as np
 
@@ -227,7 +231,7 @@ class detshuffle(PipelineStage):
     def run(self, src):
         self.epoch += 1
         rng = random.Random()
-        rng.seed((self.seed, self.epoch))
+        rng.seed(self.seed + self.epoch)
         return _shuffle(src, self.bufsize, self.initial, rng)
 
 
@@ -313,9 +317,7 @@ def _rename(data, handler=reraise_exception, keep=True, **kw):
     for sample in data:
         try:
             if not keep:
-                yield {
-                    k: getfirst(sample, v, missing_is_error=True) for k, v in kw.items()
-                }
+                yield {k: getfirst(sample, v, missing_is_error=True) for k, v in kw.items()}
             else:
 
                 def listify(v):
@@ -323,12 +325,7 @@ def _rename(data, handler=reraise_exception, keep=True, **kw):
 
                 to_be_replaced = {x for v in kw.values() for x in listify(v)}
                 result = {k: v for k, v in sample.items() if k not in to_be_replaced}
-                result.update(
-                    {
-                        k: getfirst(sample, v, missing_is_error=True)
-                        for k, v in kw.items()
-                    }
-                )
+                result.update({k: getfirst(sample, v, missing_is_error=True) for k, v in kw.items()})
                 yield result
         except Exception as exn:
             if handler(exn):
@@ -376,9 +373,7 @@ def _map_dict(data, handler=reraise_exception, **kw):
 map_dict = pipelinefilter(_map_dict)
 
 
-def _to_tuple(
-    data, *args, handler=reraise_exception, missing_is_error=True, none_is_error=None
-):
+def _to_tuple(data, *args, handler=reraise_exception, missing_is_error=True, none_is_error=None):
     """Convert dict samples to tuples."""
     if none_is_error is None:
         none_is_error = missing_is_error
@@ -387,9 +382,7 @@ def _to_tuple(
 
     for sample in data:
         try:
-            result = tuple(
-                [getfirst(sample, f, missing_is_error=missing_is_error) for f in args]
-            )
+            result = tuple([getfirst(sample, f, missing_is_error=missing_is_error) for f in args])
             if none_is_error and any(x is None for x in result):
                 raise ValueError(f"to_tuple {args} got {sample.keys()}")
             yield result
@@ -527,3 +520,172 @@ def _rsample(data, p=0.5):
 rsample = pipelinefilter(_rsample)
 
 slice = pipelinefilter(itertools.islice)
+
+
+def _extract_keys(source, *patterns, duplicate_is_error=True, ignore_missing=False):
+    for sample in source:
+        result = []
+        for pattern in patterns:
+            pattern = pattern.split(";") if isinstance(pattern, str) else pattern
+            matches = [x for x in sample.keys() if any(fnmatch("." + x, p) for p in pattern)]
+            if len(matches) == 0:
+                if ignore_missing:
+                    continue
+                else:
+                    raise ValueError(f"Cannot find {pattern} in sample keys {sample.keys()}.")
+            if len(matches) > 1 and duplicate_is_error:
+                raise ValueError(f"Multiple sample keys {sample.keys()} match {pattern}.")
+            value = sample[matches[0]]
+            result.append(value)
+        yield tuple(result)
+
+
+extract_keys = pipelinefilter(_extract_keys)
+
+
+def _rename_keys(source, *args, keep_unselected=False, must_match=True, duplicate_is_error=True, **kw):
+    renamings = [(pattern, output) for output, pattern in args]
+    renamings += [(pattern, output) for output, pattern in kw.items()]
+    for sample in source:
+        new_sample = {}
+        matched = {k: False for k, _ in renamings}
+        for path, value in sample.items():
+            fname = re.sub(r".*/", "", path)
+            new_name = None
+            for pattern, name in renamings[::-1]:
+                if fnmatch(fname.lower(), pattern):
+                    matched[pattern] = True
+                    new_name = name
+                    break
+            if new_name is None:
+                if keep_unselected:
+                    new_sample[path] = value
+                continue
+            if new_name in new_sample:
+                if duplicate_is_error:
+                    raise ValueError(f"Duplicate value in sample {sample.keys()} after rename.")
+                continue
+            new_sample[new_name] = value
+        if must_match and not all(matched.values()):
+            raise ValueError(f"Not all patterns ({matched}) matched sample keys ({sample.keys()}).")
+
+        yield new_sample
+
+
+rename_keys = pipelinefilter(_rename_keys)
+
+
+def decode_bin(stream):
+    return stream.read()
+
+
+def decode_text(stream):
+    binary = stream.read()
+    return binary.decode("utf-8")
+
+
+def decode_pickle(stream):
+    return pickle.load(stream)
+
+
+default_decoders = [
+    ("*.bin", decode_bin),
+    ("*.txt", decode_text),
+    ("*.pyd", decode_pickle),
+]
+
+
+def find_decoder(decoders, path):
+    fname = re.sub(r".*/", "", path)
+    if fname.startswith("__"):
+        return lambda x: x
+    for pattern, fun in decoders[::-1]:
+        if fnmatch(fname.lower(), pattern) or fnmatch("." + fname.lower(), pattern):
+            return fun
+    return None
+
+
+def _xdecode(
+    source,
+    *args,
+    must_decode=True,
+    defaults=default_decoders,
+    **kw,
+):
+    decoders = list(defaults) + list(args)
+    decoders += [("*." + k, v) for k, v in kw.items()]
+    for sample in source:
+        new_sample = {}
+        for path, data in sample.items():
+            if path.startswith("__"):
+                new_sample[path] = data
+                continue
+            decoder = find_decoder(decoders, path)
+            if decoder is False:
+                value = data
+            elif decoder is None:
+                if must_decode:
+                    raise ValueError(f"No decoder found for {path}.")
+                value = data
+            else:
+                if isinstance(data, bytes):
+                    data = io.BytesIO(data)
+                value = decoder(data)
+            new_sample[path] = value
+        yield new_sample
+
+xdecode = pipelinefilter(_xdecode)
+
+class Cached(PipelineStage):
+    def __init__(self):
+        super().__init__()
+        self.cached = None
+        
+    def run(self, source):
+        if self.cached is None:
+            self.temp = []
+            for sample in source:
+                self.temp.append(sample)
+                yield sample
+            self.cached = self.temp
+        else:
+            for sample in self.cached:
+                yield sample
+
+class LMDBCached(PipelineStage):
+
+    def __init__(self, fname, map_size=1e12, pickler=pickle, chunksize=500):
+        import lmdb
+        self.db = lmdb.open(fname, readonly=False, map_size=int(map_size))
+        self.pickler = pickler
+        self.chunksize = chunksize
+
+    def is_complete(self):
+        with self.db.begin() as txn:
+            return txn.get(b'_') is not None
+
+    def add_samples(self, samples):
+        with self.db.begin(write=True) as txn:
+            for key, sample in samples:
+                txn.put(key.encode(), self.pickler.dumps(sample))
+
+    def run(self, source):
+        if self.is_complete():
+            with self.db.begin(write=False) as txn:
+                for key, value in txn.cursor():
+                    if key == b"_":
+                        continue
+                    yield self.pickler.loads(value)
+        else:
+            buffer = []
+            for i, sample in enumerate(source):
+                key = (isinstance(sample, dict) and sample.get("__key__")) or str(i)
+                buffer.append((key, sample))
+                if len(buffer) >= self.chunksize:
+                    self.add_samples(buffer)
+                    buffer = []
+                yield sample
+            if len(buffer) > 0:
+                self.add_samples(buffer)
+            with self.db.begin(write=True) as txn:
+                txn.put(b'_', b'1')
\ No newline at end of file
