@@ -7,12 +7,10 @@
 
 """Open URLs by calling subcommands."""
 
-import os
-import sys
+import os, sys, re
 from subprocess import PIPE, Popen
 from urllib.parse import urlparse
 
-
 # global used for printing additional node information during verbose output
 info = {}
 
@@ -75,7 +73,10 @@ class Pipe:
             return
         self.status = self.proc.wait()
         if verbose:
-            print(f"pipe exit [{self.status} {os.getpid()}:{self.proc.pid}] {self.args} {info}", file=sys.stderr)
+            print(
+                f"pipe exit [{self.status} {os.getpid()}:{self.proc.pid}] {self.args} {info}",
+                file=sys.stderr,
+            )
         if self.status not in self.ignore_status and not self.ignore_errors:
             raise Exception(f"{self.args}: exit {self.status} (read) {info}")
 
@@ -161,11 +162,19 @@ def gopen_pipe(url, mode="rb", bufsize=8192):
     cmd = url[5:]
     if mode[0] == "r":
         return Pipe(
-            cmd, mode=mode, shell=True, bufsize=bufsize, ignore_status=[141],
+            cmd,
+            mode=mode,
+            shell=True,
+            bufsize=bufsize,
+            ignore_status=[141],
         )  # skipcq: BAN-B604
     elif mode[0] == "w":
         return Pipe(
-            cmd, mode=mode, shell=True, bufsize=bufsize, ignore_status=[141],
+            cmd,
+            mode=mode,
+            shell=True,
+            bufsize=bufsize,
+            ignore_status=[141],
         )  # skipcq: BAN-B604
     else:
         raise ValueError(f"{mode}: unknown mode")
@@ -181,12 +190,101 @@ def gopen_curl(url, mode="rb", bufsize=8192):
     if mode[0] == "r":
         cmd = f"curl -s -L '{url}'"
         return Pipe(
-            cmd, mode=mode, shell=True, bufsize=bufsize, ignore_status=[141, 23],
+            cmd,
+            mode=mode,
+            shell=True,
+            bufsize=bufsize,
+            ignore_status=[141, 23],
         )  # skipcq: BAN-B604
     elif mode[0] == "w":
         cmd = f"curl -s -L -T - '{url}'"
         return Pipe(
-            cmd, mode=mode, shell=True, bufsize=bufsize, ignore_status=[141, 26],
+            cmd,
+            mode=mode,
+            shell=True,
+            bufsize=bufsize,
+            ignore_status=[141, 26],
+        )  # skipcq: BAN-B604
+    else:
+        raise ValueError(f"{mode}: unknown mode")
+
+
+def gopen_htgs(url, mode="rb", bufsize=8192):
+    """Open a URL with `curl`.
+
+    :param url: url (usually, http:// etc.)
+    :param mode: file mode
+    :param bufsize: buffer size
+    """
+    if mode[0] == "r":
+        url = re.sub(r"(?i)^htgs://", "gs://", url)
+        cmd = f"curl -s -L '{url}'"
+        return Pipe(
+            cmd,
+            mode=mode,
+            shell=True,
+            bufsize=bufsize,
+            ignore_status=[141, 23],
+        )  # skipcq: BAN-B604
+    elif mode[0] == "w":
+        raise ValueError(f"{mode}: cannot write")
+    else:
+        raise ValueError(f"{mode}: unknown mode")
+
+
+def gopen_gsutil(url, mode="rb", bufsize=8192):
+    """Open a URL with `curl`.
+
+    :param url: url (usually, http:// etc.)
+    :param mode: file mode
+    :param bufsize: buffer size
+    """
+    if mode[0] == "r":
+        cmd = f"gsutil cat '{url}'"
+        return Pipe(
+            cmd,
+            mode=mode,
+            shell=True,
+            bufsize=bufsize,
+            ignore_status=[141, 23],
+        )  # skipcq: BAN-B604
+    elif mode[0] == "w":
+        cmd = f"gsutil cp - '{url}'"
+        return Pipe(
+            cmd,
+            mode=mode,
+            shell=True,
+            bufsize=bufsize,
+            ignore_status=[141, 26],
+        )  # skipcq: BAN-B604
+    else:
+        raise ValueError(f"{mode}: unknown mode")
+
+
+def gopen_ais(url, mode="rb", bufsize=8192):
+    """Open a URL with `curl`.
+
+    :param url: url (usually, http:// etc.)
+    :param mode: file mode
+    :param bufsize: buffer size
+    """
+    if mode[0] == "r":
+        cmd = f"ais get '{url}' -"
+        return Pipe(
+            cmd,
+            mode=mode,
+            shell=True,
+            bufsize=bufsize,
+            ignore_status=[141, 23],
+        )  # skipcq: BAN-B604
+    elif mode[0] == "w":
+        cmd = f"ais put - '{url}'"
+        return Pipe(
+            cmd,
+            mode=mode,
+            shell=True,
+            bufsize=bufsize,
+            ignore_status=[141, 26],
         )  # skipcq: BAN-B604
     else:
         raise ValueError(f"{mode}: unknown mode")
@@ -208,11 +306,33 @@ gopen_schemes = dict(
     pipe=gopen_pipe,
     http=gopen_curl,
     https=gopen_curl,
+    ais=gopen_ais,
     sftp=gopen_curl,
     ftps=gopen_curl,
     scp=gopen_curl,
+    gs=gopen_gsutil,
+    htgs=gopen_htgs,
 )
 
+if "USE_AIS_FOR" in os.environ:
+    for proto in os.environ["USE_AIS_FOR"].split(":"):
+        gopen_schemes[proto] = gopen_ais
+
+
+def rewrite_url(url):
+    name = "GOPEN_REWRITE"
+    verbose = int(os.environ.get("GOPEN_VERBOSE", 0))
+    if name not in os.environ:
+        return url
+    for r in os.environ[name].split(";"):
+        k, v = r.split("=", 1)
+        nurl = re.sub("^"+k, v, url)
+        if nurl != url:
+            if verbose:
+                print(f"GOPEN REWRITE {url} -> {nurl}")
+            return nurl
+    return url
+
 
 def gopen(url, mode="rb", bufsize=8192, **kw):
     """Open the URL.
@@ -225,9 +345,12 @@ def gopen(url, mode="rb", bufsize=8192, **kw):
 
     When no scheme is given the url is treated as a file.
 
-    You can use the OPEN_VERBOSE argument to get info about
+    You can use the GOPEN_VERBOSE argument to get info about
     files being opened.
 
+    YOu can use the USE_AIS_FOR=aws:gs:s3 to use AIS (and its cache)
+    to access urls via the "ais" command.
+
     :param url: the source URL
     :param mode: the mode ("rb", "r")
     :param bufsize: the buffer size
@@ -244,6 +367,7 @@ def gopen(url, mode="rb", bufsize=8192, **kw):
             return sys.stdout.buffer
         else:
             raise ValueError(f"unknown mode {mode}")
+    url = rewrite_url(url)
     pr = urlparse(url)
     if pr.scheme == "":
         bufsize = int(os.environ.get("GOPEN_BUFFER", -1))
