@@ -8,7 +8,7 @@
 
 
 try:
-    from torch.utils.data import IterableDataset, DataLoader
+    from torch.utils.data import DataLoader, IterableDataset
 except ModuleNotFoundError:
 
     class IterableDataset:
@@ -20,3 +20,13 @@ except ModuleNotFoundError:
         """Empty implementation of DataLoader when torch is not available."""
 
         pass
+
+
+try:
+    from torch import Tensor as TorchTensor
+except ModuleNotFoundError:
+
+    class TorchTensor:
+        """Empty implementation of TorchTensor when torch is not available."""
+
+        pass
